@@ -66,6 +66,15 @@ action_idle (gpointer user_data)
           g_object_unref (sd->screenshot);
           return FALSE;
         }
+
+      if (response == GTK_RESPONSE_REJECT)
+        {
+          g_object_unref (sd->screenshot);
+
+          /* If the user clicked 'Back' button */
+          screenshooter_region_dialog_show (sd, FALSE);
+          return FALSE;
+        }
     }
 
   if (sd->action & CLIPBOARD)
@@ -132,19 +141,25 @@ action_idle (gpointer user_data)
       if (save_location)
         {
           if (sd->action & OPEN)
-            screenshooter_open_screenshot (save_location, sd->app, sd->app_info);
+            {
+              screenshooter_open_screenshot (save_location, sd->app, sd->app_info);
+            }
           else if (sd->action & UPLOAD_IMGUR)
             {
-              if (!sd->action_specified && !screenshooter_upload_to_imgur (save_location, sd->title))
+              gboolean upload_successful = screenshooter_upload_to_imgur (save_location, sd->title);
+
+              /* If upload failed, regardless of whether it was chosen by GUI or CLI, and
+               * the action was not selected via CLI, show the actions dialog again.*/
+              if (!upload_successful && !sd->action_specified)
                 {
                   g_free (save_location);
-                  /* Show actions dialog again if no action was specified from CLI*/
                   return TRUE;
                 }
             }
         }
     }
 
+  /* Persist last used file extension */
   if (save_location)
     {
       gchar *extension = NULL;
