@@ -18,6 +18,7 @@
  */
 
 #include "screenshooter-dialogs.h"
+#include "screenshooter-actions.h"
 
 #define ICON_SIZE 16
 #define THUMB_X_SIZE 200
@@ -311,7 +312,8 @@ static void populate_liststore (GtkListStore *liststore)
   if (G_LIKELY (list_app != NULL))
     {
       g_list_foreach (list_app, (GFunc) add_item, liststore);
-      g_list_free (list_app);
+      // g_list_free (list_app);
+      g_list_free_full (list_app, g_object_unref);
     }
 }
 
@@ -513,7 +515,7 @@ save_screenshot_to_remote_location (GdkPixbuf *screenshot, GFile *save_file)
   gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
   gtk_window_set_deletable (GTK_WINDOW (dialog), FALSE);
   gtk_container_set_border_width (GTK_CONTAINER (dialog), 20);
-  gtk_window_set_icon_name (GTK_WINDOW (dialog), "document-save");
+  gtk_window_set_icon_name (GTK_WINDOW (dialog), "document-save-symbolic");
   gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), 12);
 
   gtk_label_set_markup (GTK_LABEL (label1),
@@ -612,8 +614,41 @@ cb_combo_file_extension_changed (GtkWidget *box, GtkWidget *chooser)
 
 
 
+static void
+cb_dialog_response (GtkWidget *dialog, gint response, ScreenshotData *sd)
+{
+  if (response == GTK_RESPONSE_HELP)
+    {
+      g_signal_stop_emission_by_name (dialog, "response");
+      screenshooter_open_help (GTK_WINDOW (dialog));
+    }
+  else if (response == GTK_RESPONSE_OK)
+    {
+      gtk_widget_destroy (dialog);
+      screenshooter_take_screenshot (sd, FALSE);
+    }
+  else
+    {
+      gtk_widget_destroy (dialog);
+      gtk_main_quit ();
+    }
+}
+
+
+
 /* Public */
 
+void screenshooter_region_dialog_show (ScreenshotData *sd, gboolean plugin)
+{
+  GtkWidget *dialog = screenshooter_region_dialog_new (sd, plugin);
+  g_signal_connect (dialog, "response",
+                  G_CALLBACK (cb_dialog_response), sd);
+  g_signal_connect (dialog, "key-press-event",
+                  G_CALLBACK (screenshooter_f1_key), NULL);
+  gtk_widget_show (dialog);
+  if (gtk_main_level() == 0)
+    gtk_main ();
+}
 
 
 GtkWidget *screenshooter_region_dialog_new (ScreenshotData *sd, gboolean plugin)
@@ -625,39 +660,22 @@ GtkWidget *screenshooter_region_dialog_new (ScreenshotData *sd, gboolean plugin)
   /* Create the dialog */
   if (plugin)
     {
-#if LIBXFCE4UI_CHECK_VERSION (4,14,0)
       dlg = xfce_titled_dialog_new_with_mixed_buttons (_("Screenshot"),
         NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
-        "help-browser", _("_Help"), GTK_RESPONSE_HELP,
-        "window-close", _("_Close"), GTK_RESPONSE_OK,
+        "help-browser-symbolic", _("_Help"), GTK_RESPONSE_HELP,
+        "window-close-symbolic", _("_Close"), GTK_RESPONSE_OK,
         NULL);
-#else
-      dlg = xfce_titled_dialog_new_with_buttons (_("Screenshot"),
-        NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
-        "gtk-help", GTK_RESPONSE_HELP,
-        "gtk-close", GTK_RESPONSE_OK,
-        NULL);
-#endif
 
       xfce_titled_dialog_set_subtitle (XFCE_TITLED_DIALOG (dlg), _("Preferences"));
     }
   else
     {
-#if LIBXFCE4UI_CHECK_VERSION (4,14,0)
       dlg = xfce_titled_dialog_new_with_mixed_buttons (_("Screenshot"),
         NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
-        "help-browser", _("_Help"), GTK_RESPONSE_HELP,
+        "help-browser-symbolic", _("_Help"), GTK_RESPONSE_HELP,
         "", _("_Cancel"), GTK_RESPONSE_CANCEL,
         "", _("_OK"), GTK_RESPONSE_OK,
         NULL);
-#else
-      dlg = xfce_titled_dialog_new_with_buttons (_("Screenshot"),
-        NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
-        "gtk-help", GTK_RESPONSE_HELP,
-        "gtk-cancel", GTK_RESPONSE_CANCEL,
-        "gtk-ok", GTK_RESPONSE_OK,
-        NULL);
-#endif
 
       xfce_titled_dialog_set_subtitle (XFCE_TITLED_DIALOG (dlg), _("Take a screenshot"));
     }
@@ -881,21 +899,13 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
   GdkPixbuf *thumbnail;
   GdkCursor *cursor;
 
-#if LIBXFCE4UI_CHECK_VERSION (4,14,0)
   dlg = xfce_titled_dialog_new_with_mixed_buttons (_("Screenshot"),
     NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
-    "help-browser", _("_Help"), GTK_RESPONSE_HELP,
+    "help-browser-symbolic", _("_Help"), GTK_RESPONSE_HELP,
+    "", _("_Back"), GTK_RESPONSE_REJECT,
     "", _("_Cancel"), GTK_RESPONSE_CANCEL,
     "", _("_OK"), GTK_RESPONSE_OK,
     NULL);
-#else
-  dlg = xfce_titled_dialog_new_with_buttons (_("Screenshot"),
-    NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
-    "gtk-help", GTK_RESPONSE_HELP,
-    "gtk-cancel", GTK_RESPONSE_CANCEL,
-    "gtk-ok", GTK_RESPONSE_OK,
-    NULL);
-#endif
 
   xfce_titled_dialog_set_subtitle (XFCE_TITLED_DIALOG (dlg),
                                    _("Choose what to do with the screenshot"));
@@ -1037,7 +1047,7 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
       gtk_container_add (GTK_CONTAINER (box), radio);
 
       /* Upload to imgur warning info */
-      image = gtk_image_new_from_icon_name ("dialog-warning", GTK_ICON_SIZE_BUTTON);
+      image = gtk_image_new_from_icon_name ("dialog-warning-symbolic", GTK_ICON_SIZE_BUTTON);
       label = gtk_label_new (NULL);
       gtk_label_set_markup (GTK_LABEL (label),
         _("Watch for sensitive content, the uploaded image will be publicly\n"
