@@ -40,7 +40,7 @@ create_spinner_dialog             (const gchar        *title,
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
   gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), 0);
   gtk_window_set_deletable (GTK_WINDOW (dialog), FALSE);
-  gtk_window_set_icon_name (GTK_WINDOW (dialog), "dialog-information");
+  gtk_window_set_icon_name (GTK_WINDOW (dialog), "dialog-information-symbolic");
 
   /* Create the main alignment for the dialog */
   main_alignment = gtk_box_new (GTK_ORIENTATION_VERTICAL, 1);
@@ -180,7 +180,7 @@ cb_ask_for_information (ScreenshooterJob *job,
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
   gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), 12);
 
-  gtk_window_set_icon_name (GTK_WINDOW (dialog), "gtk-info");
+  gtk_window_set_icon_name (GTK_WINDOW (dialog), "dialog-information-symbolic");
   gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
