@@ -111,34 +111,13 @@ static GOptionEntry entries[] =
 
 
 
-static void
-cb_dialog_response (GtkWidget *dialog, gint response, ScreenshotData *sd)
-{
-  if (response == GTK_RESPONSE_HELP)
-    {
-      g_signal_stop_emission_by_name (dialog, "response");
-      screenshooter_open_help (GTK_WINDOW (dialog));
-    }
-  else if (response == GTK_RESPONSE_OK)
-    {
-      gtk_widget_destroy (dialog);
-      screenshooter_take_screenshot (sd, FALSE);
-    }
-  else
-    {
-      gtk_widget_destroy (dialog);
-      gtk_main_quit ();
-    }
-}
-
-
-
 /* Main */
 
 
 
 int main (int argc, char **argv)
 {
+  ScreenshotData *sd;
   GError *cli_error = NULL;
   GFile *default_save_dir;
   const gchar *rc_file;
@@ -148,12 +127,6 @@ int main (int argc, char **argv)
     _("The --%s option is only used when --fullscreen, --window or"
       " --region is given. It will be ignored.\n");
 
-  ScreenshotData *sd = g_new0 (ScreenshotData, 1);
-  sd->plugin = FALSE;
-  sd->path_is_dir = TRUE;
-  sd->app_info = NULL;
-  sd->action = 0;
-
   xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
   /* Print a message to advise to use help when a non existing cli option is
@@ -167,7 +140,6 @@ int main (int argc, char **argv)
                    PACKAGE, cli_error->message, PACKAGE_NAME);
 
           g_error_free (cli_error);
-          g_free (sd);
 
           return EXIT_FAILURE;
         }
@@ -178,21 +150,18 @@ int main (int argc, char **argv)
     {
       g_printerr (conflict_error, "window", "fullscreen");
 
-      g_free (sd);
       return EXIT_FAILURE;
     }
   else if (window && region)
     {
       g_printerr (conflict_error, "window", "region");
 
-      g_free (sd);
       return EXIT_FAILURE;
     }
   else if (fullscreen && region)
     {
       g_printerr (conflict_error, "fullscreen", "region");
 
-      g_free (sd);
       return EXIT_FAILURE;
     }
 
@@ -201,17 +170,18 @@ int main (int argc, char **argv)
     {
       g_printerr (conflict_error, "open", "save");
 
-      g_free (sd);
       return EXIT_FAILURE;
     }
   else if (upload_imgur && (screenshot_dir != NULL))
     {
       g_printerr (conflict_error, "imgur", "save");
+
       return EXIT_FAILURE;
     }
   else if (upload_imgur && (application != NULL))
     {
       g_printerr (conflict_error, "imgur", "open");
+
       return EXIT_FAILURE;
     }
 
@@ -235,11 +205,15 @@ int main (int argc, char **argv)
     {
       g_print ("%s\n", PACKAGE_STRING);
 
-      g_free (sd);
-
       return EXIT_SUCCESS;
     }
 
+  sd = g_new0 (ScreenshotData, 1);
+  sd->plugin = FALSE;
+  sd->path_is_dir = TRUE;
+  sd->app_info = NULL;
+  sd->action = 0;
+
   /* Read the preferences */
   rc_file = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, "xfce4/xfce4-screenshooter", TRUE);
   screenshooter_read_rc_file (rc_file, sd);
@@ -268,10 +242,8 @@ int main (int argc, char **argv)
         sd->region = ACTIVE_WINDOW;
       else if (fullscreen)
         sd->region = FULLSCREEN;
-      else if (region)
-        sd->region = SELECT;
       else
-        sd->region_specified = FALSE;
+        sd->region = SELECT;
 
       /* Whether to display the mouse pointer on the screenshot */
       mouse ? (sd->show_mouse = 1) : (sd->show_mouse = 0);
@@ -328,23 +300,14 @@ int main (int argc, char **argv)
         }
 
       screenshooter_take_screenshot (sd, TRUE);
+      gtk_main ();
     }
   /* Else we show a dialog which allows to set the screenshot options */
   else
     {
-      GtkWidget *dialog;
-
-      /* Set the dialog up */
-      dialog = screenshooter_region_dialog_new (sd, FALSE);
-      g_signal_connect (dialog, "response",
-                        G_CALLBACK (cb_dialog_response), sd);
-      g_signal_connect (dialog, "key-press-event",
-                        G_CALLBACK (screenshooter_f1_key), NULL);
-      gtk_widget_show (dialog);
+      screenshooter_region_dialog_show (sd, FALSE);
     }
 
-  gtk_main ();
-
   /* Save preferences */
   screenshooter_write_rc_file (rc_file, sd);
 
